/**
 * @file board_lpcxpresso_lpc812.h
 * @brief UOS-LPC800 board abstraction layer.
 * @details UOS-LPC800.
 * @author Copyright(C) CuBeatSystems
 * @author Copyright(C) Shinichiro Nakamura
 */

#if defined(BOARD_LPCXPRESSO_LPC812)

#include "LPC8xx.h"
#include "board.h"

#define LEDBIT_G    (17)
#define LEDBIT_B    (16)
#define LEDBIT_R    (7)

#define CFG_ENABLE          (0x1 << 0)
#define CFG_DATALEN_8BIT    (0x1 << 2)

#define STAT_RXRDY          (0x1 << 0)
#define STAT_RXIDLE         (0x1 << 1)
#define STAT_TXRDY          (0x1 << 2)
#define STAT_TXIDLE         (0x1 << 3)

static int uart_init(uint32_t baudrate)
{
    uint32_t sysclk;

    LPC_SYSCON->UARTCLKDIV = 1;
    NVIC_DisableIRQ(UART0_IRQn);
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<14);
    LPC_SYSCON->PRESETCTRL &= ~(0x1<<3);
    LPC_SYSCON->PRESETCTRL |= (0x1<<3);

    sysclk = SystemCoreClock / LPC_SYSCON->UARTCLKDIV;
    LPC_USART0->CFG = CFG_ENABLE | CFG_DATALEN_8BIT;

    if (!LPC_SYSCON->UARTCLKDIV) {
        return -1;
    }

    LPC_USART0->BRG = sysclk / 16 / baudrate - 1;
    LPC_SYSCON->UARTFRGDIV = 0xFF;
    LPC_SYSCON->UARTFRGMULT = (((sysclk / 16) * (LPC_SYSCON->UARTFRGDIV + 1)) / (baudrate * (LPC_USART0->BRG + 1))) - (LPC_SYSCON->UARTFRGDIV + 1);

    LPC_USART0->STAT = STAT_RXRDY | STAT_TXRDY;
    LPC_USART0->CFG |= CFG_ENABLE;

    return 0;
}

static void set_gpio_value(uint32_t bit, uint32_t value)
{
    if (value) {
        LPC_GPIO_PORT->SET0 = (1 << bit);
    } else {
        LPC_GPIO_PORT->CLR0 = (1 << bit);
    }
}

static void set_gpio_dir(uint32_t bit, uint32_t dir)
{
    if (dir) {
        LPC_GPIO_PORT->DIR0 |=  (1 << bit);
    } else {
        LPC_GPIO_PORT->DIR0 &= ~(1 << bit);
    }
}

void board_init(void)
{
    LPC_SYSCON->SYSAHBCLKCTRL |=  (0x1 <<  6);
    LPC_SYSCON->SYSAHBCLKCTRL |=  (0x1 <<  7);
    LPC_SYSCON->PRESETCTRL    &= ~(0x1 << 10);
    LPC_SYSCON->PRESETCTRL    |=  (0x1 << 10);

    /* Pin Assign 8 bit Configuration */
    /* U0_TXD */
    /* U0_RXD */
    LPC_SWM->PINASSIGN0 = 0xffff0004UL;

    /*
     * Pin setup generated via Switch Matrix Tool
     * ------------------------------------------------
     * PIO0_5 = RESET
     * PIO0_4 = U0_TXD
     * PIO0_3 = SWDCLK
     * PIO0_2 = SWDIO
     * PIO0_1 = GPIO
     * PIO0_0 = U0_RXD
     * ------------------------------------------------
     * NOTE: LED on PIO0_2 unavailable due to SWDIO!
     * ------------------------------------------------
     */
    LPC_SWM->PINENABLE0 = 0xffffffb3UL;

    set_gpio_dir(LEDBIT_G, 1);
    set_gpio_dir(LEDBIT_B, 1);
    set_gpio_dir(LEDBIT_R, 1);

    uart_init(115200);
}

void board_led(int on)
{
    set_gpio_value(LEDBIT_G, on);
    set_gpio_value(LEDBIT_B, on);
    set_gpio_value(LEDBIT_R, on);
}

int board_uart_putc(int c)
{
    if (LPC_USART0->STAT & STAT_TXRDY) {
        LPC_USART0->TXDATA = c;
        return c;
    }
    return -1;
}

int board_uart_getc(void)
{
    if (LPC_USART0->STAT & STAT_RXRDY) {
        return LPC_USART0->RXDATA;
    }
    return -1;
}

#endif

